/***************************************************************************
                          misc.h  -  description
                             -------------------
    begin                : Sun Jul 16 2000
    copyright            : (C) 2000 by Franois Dupoux
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BITMAP_H
#define BITMAP_H

#include <stdarg.h>
#include <pthread.h>

#include "partimage.h"

//extern u32 g_dwBit[]; 
#define BIT(x) (1 << (x))
#define GET_BIT(n, bit) (!!((n) & BIT(bit)))
#define SET_BIT(n,bit,val) n = (val) ? (n | BIT(bit)) : (n & ~BIT(bit))

// =======================================================
class CBlocksUseBitmap
{
 public:
  CBlocksUseBitmap();
  ~CBlocksUseBitmap();
  
 private:
  u8 *m_cBitmap;
  u32 m_dwBitmapSize;
  u32 m_dwMaxUsedBlocksCopiedPerOper;
  u32 m_dwMaxFreeBlocksCopiedPerOper;

 public:
  int setBit(u32 dwBit, u8 bState);
  inline int isBitSet(u32 dwBit);
  
  int init(u32 dwBitmapSize);
  int freeBitmap();
  
  void setMaxUsedBlocksCopiedPerOper(u32 dwMaxUsedBlocksCopiedPerOper)
    {m_dwMaxUsedBlocksCopiedPerOper = dwMaxUsedBlocksCopiedPerOper;}
  void setMaxFreeBlocksCopiedPerOper(u32 dwMaxFreeBlocksCopiedPerOper)
    {m_dwMaxFreeBlocksCopiedPerOper = dwMaxFreeBlocksCopiedPerOper;}
  int readBitmap(u32 dwPos, u8 *bUsed, u32 *dwBitsCount);

  int fill(u8 cValue);
  u8 *data();
  u32 dataSize();
  //u32 crc(u64 qwClustersCount);
};

// =======================================================
inline int CBlocksUseBitmap::isBitSet(u32 dwBit)
{
  if (m_cBitmap == NULL)
    return -1;
  
  u32 dwLocalBit;
  u32 dwu8;
  
  dwLocalBit = dwBit % 8;
  dwu8 = (dwBit - dwLocalBit) / 8;
  
  return GET_BIT(m_cBitmap[dwu8], dwLocalBit);
  //return (!!(m_cBitmap[dwu8] & g_dwBit[dwLocalBit]));
}



#endif // BITMAP_H

