/***************************************************************************
                          cfatpart.h  -  description
                             -------------------
    begin                : Thu Jul 13 2000
    copyright            : (C) 2000 by Franois Dupoux
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FS_FAT_H
#define FS_FAT_H

#include "partimage.h"
#include "common.h"
#include "cbitmap.h"

#include "fs_base.h"

struct CImage;
struct CVolumeHeader;
struct CMainTail;
struct CMainHeader;
struct COptions;
class CSavingWindow;
class CRestoringWindow;

#define MAX_BOOT_SIZE		      4096
#define fat32to28(A) (A & 0x0FFFFFFF)

#define LOGICAL_FAT_BLKSIZE 512 // the size of the abstract block used in CFSBase

// ================================================
struct CFatFsInfoSector
{
    u32       dwMagic;          // Magic for info sector ('RRaA') 
    u8        junk[0x1dc];
    u32       reserved1;      // Nothing as far as I can tell 
    u32       signature;      // 0x61417272 ('rrAa') 
    u32       dwFreeClusters;  // Free cluster count.  -1 if unknown 
    u32       dwNextCluster;   // Most recently allocated cluster. 
    u32       reserved2[3];
    u16       reserved3;
    u16       boot_sign;
};

// ================================================
struct CInfoFatHeader // size must be 16384 (adjust the reserved data)
{
  u32 dwTotalSectorsCount;
  u32 dwClustersCount;
  u32 dwRootDirSectors;
  u32 dwRootEntriesCount;
  u32 dwSectorsPerFAT;
  u32 dwDataSectors;
  u32 dwFileSystem;
  u32 dwUsedClusters;
  u32 dwDamagedClusters;
  u32 dwFreeClusters;
  u32 dwBytesPerFatEntry;
  u32 dwReserved;		// reserved to align data
  u16 wBytesPerSector;
  u16 wReservedSectorsCount;
  u16 wRootEntriesCount;
  u8 cSectorsPerCluster;
  u8 cNumberOfFATs;
  u16 wFsInfoSector;
  u8 cReserved[16326]; // Adjust to fit with total header size
};

// ================================================
class CFatPart: public CFSBase
{
 public:
  CFatPart(char *szDevice, FILE *fDeviceFile, u64 qwPartSize);
  ~CFatPart();
  
  static  int detect(u8 *cBootSect, char *szFileSystem); 
  virtual void printfInformations();
  virtual void readBitmap(COptions *options);
  virtual void readSuperBlock();
  virtual void fsck();
  virtual void* getInfos() {return (void*)&m_info;}

  void compareFatCopies();
  int readFSInfo(u16 wFsInfoSector);

  //int checkFatDamagedClusters(u32 *dwDamagedClusters, u32 *dwUsedClusters, u32 *dwFreeClusters);
  
 private:
  CInfoFatHeader m_info;
};

#endif // FS_FAT_H
