/***************************************************************************
                          image_net.h  -  description
                             -------------------
    begin                : Mon Aug 14 2000
    copyright            : (C) 2000 by Franois Dupoux
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef IMAGE_NET_H
#define IMAGE_NET_H

#include "image_base.h"
#include "partimage.h"
#include "errors.h"
#include "messages.h"
#include "netclient.h"

#include <zlib.h> // gzip compression
#include <bzlib.h> // bzip2 compression
#include <pthread.h>

// ================================================
class CImageNet : public CImageBase
{
 public:
  CImageNet(COptions * options, u8 bUseSSL);
  virtual ~CImageNet();
  
 private:
  u8 m_bIsOpened, m_bIsSplitted;
  char m_szPath[MAXPATHLEN], m_szOriginalFilename[MAXPATHLEN];
  char m_szImageFilename[MAXPATHLEN];
  pthread_mutex_t m_mutexNetwork; // lock the buffer while working inside it
  pthread_mutexattr_t m_mutexNetworkAttr;
  
  FILE * m_fImageFile;
  //gzFile * m_gzImageFile;
  //BZFILE * m_bzImageFile;
  
 private:
  void lock(int n);
  void unlock();

 public:
  virtual u8  get_bIsOpened();
  virtual char * get_szImageFilename();
  virtual char * get_szOriginalFilename();
  virtual char * get_szPath();
  
  virtual void set_bIsOpened(u8 n);
  virtual void set_szPath(char * str);
  virtual void set_szImageFilename(char * str);
  virtual void set_szOriginalFilename(char * str);
  virtual void set_szImageFilename(char * str, int n);
  virtual void set_szOriginalFilename(char * str, int n);
  virtual void get_qwCurrentVolumeSize(u64 *qwSize);
  virtual void set_options(COptions * options);
  
 private:
  CMessages * mess;
  CNetClient * netcl;
  
 public:
  // read/write
  virtual void write(void *vBuf, u32 dwLength, int *nResult);
  //virtual void write(void *buf, u32 dwLength); // return 0 if error, or written bytes
  virtual u32 read(void *buf, u32 dwLength); // return 0 if error, or read bytes
  
  // open/close
  virtual void openWriting();
  virtual void openReading();
  virtual void close();
  virtual int mountImageLocation(char *, char *, char *);
  virtual int umountImageLocation(char *);
  
  
  // misc
  virtual void getFreeSpace(u64 * qw_AvailDiskSpace);
  virtual u8 doesFileExists(char *szPath);
  virtual void getDiskFreeSpaceForFile(u64 *qwFreeSpace, char *szFilepath);
  virtual void getFileSize(u64 *qwFileSpace, char *szFilepath);
  virtual void Connect(COptions *, char *, char *, char *, int);
  //virtual void get_qwCurVolSize(u64 *qwFileSize);
  virtual int getCompressionLevelForImage(char *szFilename);
  
};

#endif // IMAGE_NET_H
