/***************************************************************************
                          interface.h  -  description
                             -------------------
    begin                : Wed Mar 14 2001
    copyright            : (C) 2001 by Franck Ladurelle
    email                : ladurelf@partimage.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _INTERFACE_H_
#define _INTERFACE_H_

#include "partimage.h"
#include "exceptions.h"
#include "misc.h"

#define MSGBOX_YES           1
#define MSGBOX_NO            2
#define MSGBOX_CONTINUE      3
#define MSGBOX_CANCEL        4

// =======================================================
class CInterface
{
public:
  CInterface(u8 bBatchMode) {m_bBatchMode = bBatchMode;} 
  virtual ~CInterface() {} 

  u8 getBatchMode() { return m_bBatchMode; }

  // defined in herited classes
  virtual void msgBoxOk(char *szTitle, char *szText, ...)=0;
  virtual void msgBoxCancel(char *szTitle, char *szText, ...)=0;
  virtual void msgBoxError(char *szText, ...)=0;
  virtual int msgBoxYesNo(char *szTitle, char *szText, ...)=0;
  virtual int msgBoxContinueCancel(char *szTitle, char *szText, ...)=0;
  virtual void showErrorInternal(char *szText)=0;

  // defined in the base class
  void showError(signed int nErr, char *szFormat, ...);

  void showAboutDialog();

  void ErrorWriting(u32 block, signed int err);
/**/  void ErrorWritingHeader(char * header, signed int err);
/**/  void ErrorWritingBitmap(u32 block, signed int err);
/**/  void ErrorWritingDisk(u32 block, signed int err);
/**/  void ErrorWritingSuperblock(signed int err);
/**/  void ErrorWritingMBR();
/**/  void ErrorWritingMainHeader();
/**/  void ErrorWritingMainTail();
/**/  void ErrorWritingInfos();

  void ErrorReading(u32 block, signed int err);
/**/  void ErrorReadingHeader(char * header, signed int err);
  void ErrorReadingBitmap(u32 block, signed int err);
/**/  void ErrorReadingDisk(u32 block, signed int err);
  void ErrorReadingSuperblock(signed int err);
/**/  void ErrorReadingDiskBitmap(u32 block, signed int err);
/**/  void ErrorReadingMBR();
/**/  void ErrorReadingMainTail();
/**/  void ErrorReadingMainHeader();
/**/  void ErrorReadingInfos();
/**/  void ErrorReadingMBRMagic();

/**/  void ErrorNewerRelease();
/**/  void ErrorFileSystem(char *);
/**/  void ErrorDiskFull();
  void ErrorZeroing(u32 block, signed int err);

/**/  void ErrorDetectingFS(char * szDevice, signed int err);
  void ErrorEncryption();
/**/  void ErrorAskFirstVolume(char * szVolume);
/**/  void ErrorInvalidImagefile(char * szFilename);
  void ErrorTooSmall(u64 qwOriginalSize, u64 qwSize);
  
  void ErrorNoMemory();
  void ErrorAccess(char * szImageFile);
  void ErrorNoMBR();
  void ErrorCRC();
  void ErrorCRC(u64 qwOriginalCRC, u64 qwCRC);
  void ErrorWrongVolumeNumber(u32 dwExpectedVolume, u32 dwVolume);
/**/  void ErrorClosing();
  void ErrorAlreadyLocked(char * szPartition, signed int err);
  void ErrorOpeningPartition(char * szDevice, signed int err);
  void ErrorSavingMBR();
/**/  void ErrorInvalidFS(char * szFs);
/**/  void ErrorLocking(char * szFilename, signed int err);

  virtual void StatusLine(char * str)=0;
  virtual int askLogin(char * szLogin, char * szPasswd, u16 size)=0;
  virtual u16 askNewPath(char * szOriginalFilename, u32 dwVolume,
     char * szPath, char *szNewPath, u16 size)=0;
  virtual int askText(char * szMessage, char * szTitle, char * szDestText,
     u16 size)=0;
  virtual void askDescription(char * szDescription, u32 dwMaxLen)=0;
  virtual int guiInitMainWindow(char * szDevice, char * szImageFile, char *
     szNetworkIP, u32 * dwServerPort, u8 * bSsl)=0;
  /*virtual int askText(char * szMessage, char * szTitle, char * szDestText)=0;
  virtual void showAboutDialog()=0;
  virtual u16 windowRoot()=0;
  virtual u16 WaitKeyPressed(char * szOld, char * szNewPath,
     char * szOriginalFilename, u32 volume)=0;
  virtual u16 askRestore(char * szDevice, char * szImageFilename)=0;
  virtual u16 askIgnoreFSError(char * fsck)=0;*/

//  u16 Error(CExceptions * excep, char * szFilename);

  u16 ErrorLogAsRoot();
  u16 ErrorBugBzip2();
  u16 WaitKeyPressed(char *szOld, char *szNewPath, char *szOriginalFilename, u32 dwVolume);
  u16 askRestore(char *szDevice, char *szImageFilename);
  u16 askIgnoreFSError(char *szFsck);
  u16 askIgnoreNoFschk(char *szFsck);
  u16 askIgnoreDeniedFschk(char *szFsck);
  void WarnFS(char *szFileSys);
  void WarnFsBeta(char *szFileSys);
  u16 WarnNotSimulate();
  u16 WarnRestoreMBR(char * szCurrentDevice, u64 qwCurrentSize, char * szOriginalDevice, u64 qwOriginalSize);
  u16 WarnRestoreOtherMBR(char *szCurrentDevice,  char *szOriginalDevice);
  void SuccessRestoringMBR(char *szDevice);
  u16 Error(CExceptions *excep, char *szFilename, char *szDevice=NULL);
  //void Error();

private:
  u8 m_bBatchMode;
};

#endif
