#!/bin/sh

#
# Copyright 2005 Franck Ladurelle <ladurelf@partimage.org>
# Released under GNU GPL 2.
#


#
# Ce script ncessite les excutables createdata et aide.
# TEST_PART est la partition de test. Elle sera successivement efface,
#   remplie, sauvegarde, efface, restaure et vrifie.
# TMP_PART est la partition qui contiendra les images faites par partimage.
#   sa taille devrait tre suprieure  celle de TEST_PART.
#


AIDE=/usr/bin/aide
AIDECONF=./aide.conf

TEST_PART=/dev/hdaX
TMP_PART=/dev/hdaY
LOG=/tmp/regression.log
FS="ext2 ext3 reiserfs vfat xfs"

#

mkdir -p /mnt/tmp /mnt/tests
mount $TMP_PART /mnt/tmp
rm -f $LOG
touch $LOG

mk_fs()
{
	case "$1" in
		ext2) mkfs.ext2 $2;;
		ext3) mkfs.ext3 $2;;
		reiserfs) mkfs.reiserfs -f $2;;
		vfat) mkfs.vfat -F 32 $2;;
		xfs) mkfs.xfs -f $2;;
		*) echo "$1 non supporte"; exit 1;;
	esac
	return 0
}

for fs in $FS ; do
	for n in 100 50 25 10 5 3 2 ; do
		echo "effacement ..."
		dd if=/dev/zero of=$TEST_PART
		echo "formatage ..."
		mk_fs $fs $TEST_PART
		mount $TEST_PART /mnt/tests
		echo "creation des fichiers ..."
		createdata -d $n /mnt/tests
		rm -f /tmp/aide.db /tmp/aide.db.new
		echo "calcul CRC ..."
		$AIDE -c $AIDECONF --init
		umount /mnt/tests
		mv /tmp/aide.db.new /tmp/aide.db
		rm -f /mnt/tmp/test.img*
		partimage -bcd save $TEST_PART /mnt/tmp/test.img
		if [ "$?" = 0 ]; then
			echo "sauvegarde ok pour [$fs:$n]" >> $LOG
			echo "effacement ..."
			dd if=/dev/zero of=$TEST_PART
			partimage -g1 -b restore $TEST_PART /mnt/tmp/test.img.000
			if [ "$?" = 0 ]; then
				echo "restauration ok pour [$fs:$n]" >> $LOG
				echo "verification CRC ..."
				mount $TEST_PART /mnt/tests
				$AIDE -c $AIDECONF --check | grep "Looks okay" > /dev/null
				if [ "$?" = 0 ]; then
					echo "CRC ok pour [$fs:$n]" >> $LOG
				else
					echo "erreur CRC pour [$fs:$n]" >> $LOG
				fi
				umount /mnt/tests
			else
				echo "echec restauration pour [$fs:$n]" >> $LOG
			fi
		else
			echo "echec sauvegarde pour [$fs:$n]" >> $LOG
		fi
	done
done




umount /mnt/tmp

exit 0
